local game = {}

function game:load()
	--TEXT OBJECTS
	local verb = "click or press space"
	local escVerb = "press escape"
	if OS == "Android" or OS == "iOS" then
		verb = "tap"
		escVerb = "tap back"
	end
	self.startMessage = text.new(string.format("%s to start", verb), 0, math.floor(screen.height * 0.4), "center", FONT.small, self.textColorAccent, "bottom")
	self.startMessage:hide(true)

	self.pauseMessage = text.new("paused\n\n"..verb.." to resume, "..escVerb.." to quit", 0, math.floor(screen.height * 0.4), "center", FONT.small, self.textColor, "bottom")
	self.pauseMessage:hide(true)

	self.lostText = "GAME OVER\nscore: %d\n\n"..verb.." to restart"
	self.lostMessage = text.new(self.lostText, 0, math.floor(screen.height * 0.4), "center", FONT.small, self.textColorAccent, "bottom")
	self.lostMessage:hide(true)

	self.scoreMessage = text.new("0", 0, math.floor(screen.height * 0.05), "center", FONT.large, self.textColor, "top")
	self.scoreMessage:hide(true)

	self.highScoreMessage = text.new("highscore: "..CONFIG.game.highScore, -math.floor(screen.width * 0.01), math.floor(screen.height * 0.93), "right", FONT.small, self.textColor, "bottom")
	self.highScoreMessage:hide(true)

	--Screen cover
	self.cover = {
		height = screen.height,
		dHeight = screen.height,
		alpha = 0,
		dAlpha = 0,
		onAlpha = 80
	}

	self.restartDelay = 1
	self.canRestart = true

	platform:load()
	player:load()
	game:reset()
end

function game:showCover()
	--self.cover.height = screen.height
	self.cover.alpha = self.cover.onAlpha
end

function game:hideCover()
	--self.cover.height = 0
	self.cover.alpha = 0
end

function game:reset()
	if self.canRestart then
		self.score = 0
		self.scoreMessage:setText(self.score)
		self.restartTick = 0
		self.paused = false
		self.started = false
		self.lost = false
		player:reset()
		platform:clear()
		platform:spawn()
		self.startMessage:show()
		self.lostMessage:hide()
		self.scoreMessage:show()
		self.highScoreMessage:show()
		game:showCover()
	end
end

function game:lose()
	self.lostMessage:setText(string.format(self.lostText, self.score))
	player.alpha = 0
	platform:hideAll()
	self.lostMessage:show()
	self.scoreMessage:hide()
	self.highScoreMessage:hide()
	self:showCover()
	self.canRestart = false
	self.lost = true
end

function game:update(dt)
	--Restart delay
	if self.lost then
		if not self.canRestart then
			self.restartTick = self.restartTick + dt
			if self.restartTick > self.restartDelay then
				self.canRestart = true
			end
		end
	end

	--Cover
	self.cover.dHeight = self.cover.dHeight + (self.cover.height - self.cover.dHeight) * (SMOOTH_FACTOR * dt)
	self.cover.dAlpha = self.cover.dAlpha + (self.cover.alpha - self.cover.dAlpha) * (SMOOTH_FACTOR * dt)

	--Text
	self.startMessage:update(dt)
	self.pauseMessage:update(dt)
	self.lostMessage:update(dt)
	self.scoreMessage:update(dt)
	self.highScoreMessage:update(dt)

	player:update(dt)
	platform:update(dt)

	--Collision
	local colPlatforms = {
		platform.array[1],
		platform.array[2]
	}
	for i,v in ipairs(colPlatforms) do
		if AABB(player.x, player.y, player.size, player.size, v.x, 0, platform.width, v.topY) or AABB(player.x, player.y, player.size, player.size, v.x, v.bottomY, platform.width, (screen.height - v.bottomY)) then
			self:lose()
		end
	end

	--Score count
	for i,v in ipairs(platform.array) do
		if v.x + platform.width < player.x then
			if not v.scored then
				self.score = self.score + 1
				self.scoreMessage:setText(self.score)
				if self.score > CONFIG.game.highScore then
					CONFIG.game.highScore = self.score
					self.highScoreMessage:setText("highscore: "..self.score)
					ini.save("config.ini", CONFIG)
				end
				v.scored = true
			end
		end
	end
end

function game:draw()
	player:draw()
	platform:draw()

	self.scoreMessage:draw()
	self.highScoreMessage:draw()

	--COVER
	love.graphics.setColor(self.coverColor)
	setAlpha(self.cover.dAlpha)
	love.graphics.rectangle("fill", 0, 0, screen.width, self.cover.dHeight)

	self.startMessage:draw()
	self.pauseMessage:draw()
	self.lostMessage:draw()

	--if not self.started then
	--	love.graphics.setColor(self.textColorAccent)
	--	love.graphics.printf(self.startMessage, 0, math.floor(screen.height * 0.3), screen.width, "center") 
	--end
end

function game:flap()
	if not self.started then
		self.started = true
		self.startMessage:hide()
		self.scoreMessage:show()
		self.highScoreMessage:show()
		self:hideCover()
	end
	if self.lost then
		self:reset()
	end
	if self.paused then
		self.paused = false
		self.pauseMessage:hide()
		self:hideCover()
	end

	player:flap()
end

function game:keypressed(key)
	if key == "space" then
		self:flap()
	elseif key == "escape" then
		if self.started and not self.lost then
			if self.paused then
				love.event.push("quit")
			else
				self.paused = true
				self.pauseMessage:show()
				self:showCover()
			end
		end
	end
end

function game:mousepressed(x, y, key)
	if key == 1 then
		self:flap()
	end
end

function game:touchpressed(id, x, y, dx, dy, pressure)
	self:flap()
end

return game